/*
 * =======================================================================
 * ============================ JAR WIZARD ===============================
 * =======================================================================
 *
 *                                                 Copyright (C) 2002 Chir
 *
 * DESCRIPTION:   This Swing program helps packing a Java application
 *                into a JAR file that then launches with 2 clicks.
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 *
 * YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
 * ALONG WITH THIS PROGRAM; IF NOT, WRITE TO THE FREE SOFTWARE
 * FOUNDATION, INC., 59 TEMPLE PLACE, SUITE 330, BOSTON, MA 02111-1307 USA
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.lang.reflect.*;

/**
 * Window container and launcher, for console-based applications.
 *
 * @author  Chir
 */
public class JConsole extends JFrame implements WindowListener, Runnable
{
    /** Console output re-direction */
    class ConsoleByteArrayOutputStream extends ByteArrayOutputStream
    {
        public void write( byte[] b, int off, int len )
        {
            super.write( b, off, len );
            m_textArea.setText( m_textArea.getText() + new String(b,off,len) );   // Echo to the console.
        }
    }
    /** Console input re-direction */
    class ConsoleByteArrayInputStream extends ByteArrayInputStream
    {
        public ConsoleByteArrayInputStream()
        {
            super( new byte[0] );
        }
        public int read( byte[] b, int off, int len )
        {
            int r = super.read( b, off, len );
            String res;

            do
            {
                res = JOptionPane.showInputDialog( null );
            } while( res == null );

            res += "\n";
            if( res.length() > len - off )  res = res.substring( 0, len - off );

            System.arraycopy( res.getBytes(), 0, b, off, res.length() );
            m_textArea.setText( m_textArea.getText() + res );   // Echo to the console.

            return res.length();
        }
    }

    public JConsole()
    {
        m_textArea = new JTextArea( 25, 80 );
        m_textArea.setFont( new Font( "Courier", Font.PLAIN, 12 ) );
        m_textArea.setEditable( false );
        m_textArea.setForeground( Color.cyan );  m_textArea.setBackground( Color.black );

        m_scrollPane = new JScrollPane();
        m_scrollPane.setPreferredSize( new java.awt.Dimension(600, 450) );
        m_scrollPane.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
        m_scrollPane.setViewportView( m_textArea );
        getContentPane().add( m_scrollPane, BorderLayout.CENTER );

        addWindowListener( this );

        pack();
        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        int x0 = (screenDim.width - getSize().width) / 2 , y0 = (screenDim.height - getSize().height) / 2;
        setLocation( x0, y0 );  // Center on screen.

        // --- Set the System out and err streams to use our replacements ---
        System.setIn( new ConsoleByteArrayInputStream() );                      // Create a new input stream for the standard input.
        System.setOut( new PrintStream(new ConsoleByteArrayOutputStream()) );   // Create a new output stream for the standard output.
        System.setErr( new PrintStream(new ConsoleByteArrayOutputStream()) );   // Create new output stream for the standard error output.

        // --- Determine the class name and the function to call ---
        try
        {
            BufferedReader br = new BufferedReader( new InputStreamReader(getClass().getResourceAsStream("main.cfg")) );
            m_mainClassName = br.readLine();
        }
        catch( Exception e )
        {
            JOptionPane.showMessageDialog( null, "The file 'main.cfg' is missing.", "Error", JOptionPane.ERROR_MESSAGE );
            System.exit( 1 );
        }

        m_fn = null;

        try
        {
            Class c = Class.forName( m_mainClassName );
            try
            {
                Class[] parameterTypes = { String[].class };
                m_fn = c.getMethod( "main", parameterTypes );
            }
            catch( java.lang.Throwable e )
            {
                JOptionPane.showMessageDialog( null, "Class '" + m_mainClassName + "' can't be started\nbecause it has no main function.", "Error", JOptionPane.ERROR_MESSAGE );
                System.exit( 1 );
            }
        }
        catch( java.lang.Throwable e )
        {
            JOptionPane.showMessageDialog( null, "Can't find class '" + m_mainClassName + "'", "Error", JOptionPane.ERROR_MESSAGE );
            System.exit( 1 );
        }

        Thread th = new Thread( this );
        th.start();                     // Call main()
    }

    public void run()
    {
        setTitle( m_mainClassName + " - running" );
        m_textArea.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );
        try
        {
            String[] argv0 = { };
            Object[] argv = { argv0 };
            m_fn.invoke( null, argv );  // Call main() without parameters.
        }
        catch( java.lang.Throwable e )
        {
            JOptionPane.showMessageDialog( null, "Failed to launch '" + m_mainClassName + "'", "Error", JOptionPane.ERROR_MESSAGE );
            System.exit( 1 );
        }
        m_textArea.setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
        setTitle( m_mainClassName + " - finished" );
    }

    public void windowClosing( WindowEvent e )      { System.exit(0); }
    public void windowOpened( WindowEvent e )       { }
    public void windowClosed( WindowEvent e )       { }
    public void windowIconified( WindowEvent e )    { }
    public void windowDeiconified( WindowEvent e )  { }
    public void windowActivated( WindowEvent e )    { }
    public void windowDeactivated( WindowEvent e )  { }
    
    public static void main( String args[] )
    {
        new JConsole().setVisible( true );
    }

    private JTextArea   m_textArea;
    private JScrollPane m_scrollPane;
    private String      m_mainClassName;
    private Method      m_fn;                            // Temporary pointer to the "main" function to launch.
}
